@@ -258,6 +258,10 @@ class Agent(Insight, Channel, Catalog, Dataset):
 
     def create_dataset(self, dataframe:pd.DataFrame, dataset_name:str, slug: str, settings: dict = None):
 
+        for col in dataframe.columns:
+            if (dataframe[col].dtypes == "datetime64[ns]"):
+                dataframe[col] = dataframe[col].astype("string")
+
         body = {"label": dataset_name, "rows": dataframe.to_dict(orient="record")}
 
         settings_list = []
@@ -280,12 +284,17 @@ class Agent(Insight, Channel, Catalog, Dataset):
             "Content-Type": "application/json",
             "Authorization": "Bearer" + " " + self._token
         }
+
         response = s.post(url=url, json=body, headers=headers)
         response.raise_for_status()
 
     def update_dataset(self, dataframe: pd.DataFrame, dataset_name: str, slug: str, settings: dict = None):
 
-        body = {"label": dataset_name, "rows": dataframe.to_dict(orient="record")}
+        for col in dataframe.columns:
+            if (dataframe[col].dtypes == "datetime64[ns]"):
+                dataframe[col] = dataframe[col].astype("string")
+
+        body = {"label": dataset_name, "rows": dataframe.to_dict(orient="records")}
 
         settings_list = []
         if (settings != None):
@@ -307,7 +316,7 @@ class Agent(Insight, Channel, Catalog, Dataset):
             "Content-Type": "application/json",
             "Authorization": "Bearer" + " " + self._token
         }
-        print(body)
+
         response = s.put(url=url, json=body, headers=headers)
 
         response.raise_for_status()
@@ -332,7 +341,7 @@ class Agent(Insight, Channel, Catalog, Dataset):
             "DefaultAggreation": "sum"
         }
     }
-'''
+    '''
 
 
     def create_or_replace_dataset(self, dataframe:pd.DataFrame, dataset_name:str, slug:str, settings:dict = None):
@@ -360,7 +369,17 @@ class Agent(Insight, Channel, Catalog, Dataset):
         else:
             raise Exception('takes 2 positional arguments "slug, datset_id" but 0 were given')
 
-    def create_datacard(self, channel: str, title: str,  search:str = "", slug: str = None):
+    def create_datacard(self, channel: str, title: str,  search:str = "", slug: str = None, replace:bool = False):
+
+        if(replace==True and slug!= None):
+            try:
+                datacard = self.get_datacard(slug)
+                datacard.delete()
+            except:
+                pass
+        elif(replace==True and slug!= None):
+            print("Please specify a datacard slug to replace!")
+            return
 
         channel = self.get_channel(self._agentId, channel)
         if channel!=None:
@@ -422,7 +441,7 @@ class Agent(Insight, Channel, Catalog, Dataset):
 
     def get_datacard(self, slug):
 
-        url = self.smart_insight_url+"definitions/agent/"+self._agentId+"/slug/"+slug
+        url = self.smart_insight_url+"/definitions/agent/"+self._agentId+"/slug/"+slug
 
         s = requests.Session()
         s.keep_alive = False
