@@ -1,8 +1,6 @@
 import requests
 import yaml
 import os
-import pandas as pd
-import numpy as np
 import logging
 from askdata.insight import Insight
 from askdata.channel import Channel
@@ -39,19 +37,22 @@ class Insight_Definition:
         self.name = defintion["name"]
         self.slug = defintion["slug"]
         self.icon = defintion["icon"]
-        self.components = defintion["components"]
+        self.components: list = defintion["components"]
 
         self._token = token
 
         if env.lower() == 'dev':
             self._base_url_askdata = url_list['BASE_URL_ASKDATA_DEV']
             self.smart_insight_url = url_list['BASE_URL_INSIGHT_DEV']
+            self.app_url = "https://app-dev.askdata.com/"
         if env.lower() == 'qa':
             self._base_url_askdata = url_list['BASE_URL_ASKDATA_QA']
             self.smart_insight_url = url_list['BASE_URL_INSIGHT_QA']
+            self.app_url = "https://app-qa.askdata.com/"
         if env.lower() == 'prod':
             self._base_url_askdata = url_list['BASE_URL_ASKDATA_PROD']
             self.smart_insight_url = url_list['BASE_URL_INSIGHT_PROD']
+            self.app_url = "https://app.askdata.com/"
 
 
     def add_table(self, query="", columns=[]):
@@ -197,11 +198,13 @@ class Insight_Definition:
             "id": text_id,
             "name": name,
             "queryComponent": False,
-            "text": "Ciao",
-            "type": text,
+            "text": text,
+            "type": "text",
             "valid": True
         }
 
+        logging.info("URL: {}".format(url))
+
         s = requests.Session()
         s.keep_alive = False
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
@@ -281,11 +284,11 @@ class Insight_Definition:
         map_id = self.add_component("map", position)
         return map_id
 
-    def add_sql_query(self, query_sql, dataset_slug):
+    def add_sql_query(self, query_sql, dataset_slug, is_native = False):
 
         position = (len(self.components))
 
-        self.add_component("sql_query", position)
+        sql_id = self.add_component("sql_query", position)
 
         s = requests.Session()
         s.keep_alive = False
@@ -299,13 +302,37 @@ class Insight_Definition:
         response = s.get(url=authentication_url, headers=headers)
         response.raise_for_status()
         r = response.json()
-        dataset_id = r.json()["dataset"]["id"]
+        dataset_id = r["dataset"]["id"]
+
+        url_put = self.smart_insight_url + "/definitions/" + self.definition_id + "/sql_queries/" + sql_id
+        body2 = {
+            "datasetId": dataset_id,
+            "sql": query_sql,
+            "id": sql_id,
+            "nativeType": is_native,
+            "queryComponent": True,
+            "type": "sql_query",
+            "valid": True,
+            "variableName": sql_id + "Result"
+        }
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.put(url=url_put, json=body2, headers=headers)
+        r.raise_for_status()
 
         url = self.smart_insight_url+"/definitions/"+self.definition_id+"/sql_queries/"+self.components[position]["id"]+"/sql"
 
         body = {
             "datasetId": dataset_id,
-            "sql": query_sql
+            "sql": query_sql,
         }
 
         s = requests.Session()
@@ -320,9 +347,19 @@ class Insight_Definition:
 
         r = s.put(url=url, json=body, headers=headers)
         r.raise_for_status()
+
         self.components = r.json()["components"]
 
-        return self.components[position]["id"]
+        hasResultSet = True
+
+        for component in r.json()["components"]:
+            if component["id"] == sql_id:
+                if component["preview"] == []:
+                    hasResultSet = False
+                elif component["preview"][0]["details"]["rows"] == []:
+                    hasResultSet = False
+
+        return sql_id , hasResultSet
 
 
     def delete(self):
@@ -438,8 +475,6 @@ class Insight_Definition:
         r = response.json()
 
 
-
-
     def add_search_query(self, query):
 
         position = (len(self.components))
@@ -465,6 +500,7 @@ class Insight_Definition:
 
         return self.components[position]["id"]
 
+
     def add_component(self, type, position):
 
         body = {"type": type, "position": position}
@@ -486,7 +522,95 @@ class Insight_Definition:
         self.components = r.json()["components"]
         return self.components[position]["id"]
 
+
+    def get_url(self):
+        url = self.app_url
+
+        get_url = self._base_url_askdata+"/smartbot/agents/"+self.agent_id
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.get(url=get_url, headers=headers)
+        r.raise_for_status()
+
+        agent_slug = r.json()["slug"]
+
+        url += agent_slug
+
+        url += "/card/"
+        url += self.slug
+        print(url)
+        return url
+
+    def edit_card(self, icon="", name=""):
+
+        if(icon==""):
+            icon = self.icon
+        else:
+            self.icon=icon
+
+        if(name==""):
+            name = self.name
+        else:
+            self.name=name
+
+        body={
+            "icon": icon,
+            "name": name,
+            "slug": self.slug
+        }
+
+        url = self.smart_insight_url+"/definitions/"+self.definition_id
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.put(url=url, json=body, headers=headers)
+        r.raise_for_status()
+        self.components = r.json()["components"]
+
+
+    def get(self):
+
+        get_url = self.smart_insight_url+"/definitions/"+self.definition_id+"/card"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.get(url=get_url, headers=headers)
+        r.raise_for_status()
+
+
+        return r.json()["attachment"]
+
     def delete_component(self, component_id):
+
+        for component in self.components:
+            if(component["id"]==component_id):
+                self.components.remove(component)
+
         s = requests.Session()
         s.keep_alive = False
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
@@ -501,9 +625,34 @@ class Insight_Definition:
         r = s.delete(url=query_url, headers=headers)
         r.raise_for_status()
 
-    def publish(self):
+    def schedule_daily(self, channel_slug, hours, minutues, seconds):
+
+        cron = seconds + " " + minutues + " " + hours + " * * *"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+        url = self._base_url_askdata + '/smartfeed/channels?agentId=' + self.agent_id + '&slug=' + channel_slug
+        r = s.get(url=url, headers=headers)
+        r.raise_for_status()
+
+        channel_id = r.json()[0]["id"]
+
+        url = self.smart_insight_url + "/definitions/" + self.definition_id + "/publishing/"
+
+        body = {
+            "schedulingCron": cron,
+            "enabled": True,
+            "destination": {"channels":[channel_id],"code":None,"name":None,"icon":None,"visibility":None,"queryId":None,"empty":None},
+            "condition": "always",
+            "advancedConditions": None}
 
-        url = self.smart_insight_url + "/definitions/" + self.definition_id + "/publish/"
 
         s = requests.Session()
         s.keep_alive = False
@@ -515,5 +664,93 @@ class Insight_Definition:
             "Authorization": "Bearer" + " " + self._token
         }
 
-        r = s.post(url=url, headers=headers)
+        r = s.post(url=url, json=body, headers=headers)
+        r.raise_for_status()
+
+
+    def schedule_monthly(self, channel_slug, day_of_month, hours, minutues, seconds):
+
+        """day_of_week: day of the week in format MON, TUE, WED, THU, FRI, SAT, SUN"""
+
+        cron = seconds + " " + minutues + " " + hours + day_of_month + " * ?"
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+        url = self._base_url_askdata + '/smartfeed/channels?agentId=' + self.agent_id + '&slug=' + channel_slug
+        r = s.get(url=url, headers=headers)
+        r.raise_for_status()
+
+        channel_id = r.json()[0]["id"]
+
+        url = self.smart_insight_url + "/definitions/" + self.definition_id + "/publishing/"
+
+        body = {
+            "schedulingCron": cron,
+            "enabled": True,
+            "destination": {"channels":[channel_id],"code":None,"name":None,"icon":None,"visibility":None,"queryId":None,"empty":None},
+            "condition": "always",
+            "advancedConditions": None}
+
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.post(url=url, json=body, headers=headers)
+        r.raise_for_status()
+
+    def schedule_weekly(self, channel_slug, day_of_week, hours, minutues, seconds):
+        """day_of_week: day of the week in format MON, TUE, WED, THU, FRI, SAT, SUN"""
+
+        cron = seconds + " " + minutues + " " + hours + " ? * " + day_of_week
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+        url = self._base_url_askdata + '/smartfeed/channels?agentId=' + self.agent_id + '&slug=' + channel_slug
+        r = s.get(url=url, headers=headers)
+        r.raise_for_status()
+
+        channel_id = r.json()[0]["id"]
+
+        url = self.smart_insight_url + "/definitions/" + self.definition_id + "/publishing/"
+
+        body = {
+            "schedulingCron": cron,
+            "enabled": True,
+            "destination": {"channels": [channel_id], "code": None, "name": None, "icon": None, "visibility": None,
+                            "queryId": None, "empty": None},
+            "condition": "always",
+            "advancedConditions": None}
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+
+        r = s.post(url=url, json=body, headers=headers)
         r.raise_for_status()
